/* Given a sequence u(n) as a closure with asymptotic expansion
 * u(n) = a0 + a1/n^al + a2/n^{2al} + ... Find a0, ..., a_LIM
 * where the a_i are left as floating point numbers. */

/* vu(N) = [u_1, ..., u_N] */
Asymprawvec(vu, al = 1, LIM = 10) =
{ my(B = getlocalbitprec(), v = vector(LIM+1));
  my(c = LimitmulN(al), d = Limitaccuracy(al));
  my(N, B1, W, U, R);

  localbitprec(32);
  N = ceil(Solvedivlog(c * al * LIM / log(2), c * B));
  B1 = ceil(B + N / c + d * N);
  localbitprec(B1); al = bitprecision(al, B1);
  W = Limitweights(N, al);
  U = vu(N);
  R = vectorv(N, n, n^al);
  for (k = 0, LIM,
    my(z = sum(i = 1, #W, W[i] * U[i]));
    U = vectorv(#U, n, bitprecision((U[n] - z) * R[n], B1));
    v[k+1] = bitprecision(z, B));
  return (v);
}
/* u(n) = u_n */
Asympraw(u, al = 1, LIM = 10) =
  Asymprawvec(N -> vector(N, n, u(n)), al, LIM);
