/* Given a continued fraction CF output by the Quodif program,
 * convert it into an Euler type continued fraction A(n),B(n),
 * where
 * b[1]z / (1 + b[2]z / (1 + b[3]z / (1 + ... + b[lim]z)))
 * = B[1] / (1/z + A[1] + B[2] / (1/z + A[2]
 *   + B[3] / (1/z + ... + B[lim\2] / (1/z + A[lim\2])))). */
CFtoEuler(CF) =
{ my(A = vector(#CF \ 2), B = vector(#A));

  for (i = 2, #A,
    A[i] = CF[2*i] + CF[2*i-1];
    B[i] = -CF[2*i-1] * CF[2*i-2]);
  A[1] = CF[2];
  B[1] = CF[1]; return ([A, B]);
}

/* Given a continued fraction CFAB output by CFtoEuler,
 * evaluate its first N terms at 1 / t. */
CFEulereval(CFAB, N, t) =
{ my([A, B] = CFAB, S = 0);

  if (#A < N, error("short continued fraction"));
  if (N%2, S = B[N] / (A[N] + t); N--);
  forstep (j = N, 2, -2,
    S += A[j] + t;
    S = B[j-1] * S / ((A[j-1] + t) * S + B[j]));
  return (S);
}
