/* Let [F,E,D,P] code an asymptotic expansion of type
 * n!^F * E^n * e^{(D*n)^{1/2}} * n^P.
 * Compute value at n. */

CFtypeeval(FEDP, n) =
{ my(S, [f, e, d, p] = FEDP());

  S = if (f, factorial(n)^f, 1);
  if (e != 1, S *= e^n);
  if (d, S *= exp(sqrt(d * n)));
  if (p, S *= n^p);
  return (S);
}

/* Let a(0)+b(0)/(a(1)+b(1)/(a(2)+...)) be a continued
 * fraction; compute [[p1,q1], ..., [pn,qn]]. */
CFpnqn(a, b, N) =
{ my(V = vector(N));

  V[1] = [a(0) * a(1) + b(0) * 1., a(1) * 1.];
  V[2] = a(2) * V[1] + b(1) * [a(0), 1];
  for (n = 2, N-1, V[n+1] = a(n+1) * V[n] + b(n) * V[n-1]);
  return (V);
}

/* Compute limit. */
CFlim(a, b, al = 1/2, mul = 10) =
{ my(VU);
  VU = (N -> my(V = CFpnqn(a, b, mul * N));
             vector(N, n, my([pn, qn] = V[mul * n]); pn / qn));
  Limitvec(VU, al);
}

/* f(n) known function such that q(n) is asymptotic to
 * C.[F,E,D,P](n) (syntax as above) for some constant C
 * with asymptotic behavior in integral powers of 1/n^al.
 * Compute C. All parameters MUST be given as closures. */
CFqnasymp(a, b, FEDP, al = 1/2, mul = 10) =
{ my(VU);

  VU = (N -> my(V = CFpnqn(a, b, mul * N));
    vector(N, n, my(qn = V[mul * n][2]);
                 qn / CFtypeeval(FEDP, mul * n)));
  Limitvec(VU, al);
}

/* Assume known limit S, and f(n) a known function such that
 * S-p(n)/q(n) is asymptotic to C/f(n) = C/[F,E,D,P](n)
 * (syntax as above) for some constant C with asymptotic
 * behavior in integral powers of 1/n^al. Compute C.
 * All parameters MUST be given as closures. */
CFasymp(a, b, FEDP, S, al = 1/2, mul = 10) =
{ my(VU);

  VU = (N -> my(B = getlocalbitprec());
    localbitprec(32);
    localbitprec(B + exponent(CFtypeeval(FEDP, mul * N)));
    my(s = S(), V = CFpnqn(a, b, mul * N));
    vector(N, n, my([pn,qn] = V[mul * n]);
                 CFtypeeval(FEDP, mul * n) * (s - pn / qn)));
  Limitvec(VU, al);
}
