/* Compute S(n) using recursions for p(n) and q(n). Output
 * [S(1)...S(N)] if 'all' is set, otherwise S(N). */
CFforward0(a, b, N, all = 0) =
{ my(pm1 = 1., p0 = a(0) * 1., qm1 = 0., q0 = 1., v);

  if (all, v = vector(N));
  for (n = 1, N,
    my(A = a(n), B = b(n - 1));
    my(p1 = A * p0 + B * pm1); pm1 = p0; p0 = p1;
    my(q1 = A * q0 + B * qm1); qm1 = q0; q0 = q1;
    if (all, v[n] = p0 / q0));
  return (if (all, v, p0 / q0));
}

/* Compute S(n) using the formula for S(n+1) - S(n). Output
 * [S(1)...S(N)] if 'all' is set, otherwise S(N). */
CFforward1(a, b, N, all = 0) =
{ my(qm1 = 0., S = a(0) * 1., q0 = 1., prodb = 1., v);

  if (all, v = vector(N));
  for (n = 0, N-1,
    my(B = b(n));
    my(q1 = a(n+1) * q0 + B * qm1); qm1 = q0; q0 = q1;
    prodb *= B;
    S += (-1)^n * prodb / (q0 * qm1);
    if (all, v[n+1] = S));
  return (if (all, v, S));
}

/* Compute S(2n) using the formula for S(2n+2) - S(2n). Output
 * [S(2)...S(2N)] if 'all' is set, otherwise S(2N). */
CFforward2(a, b, N, all = 0) =
{ my(qm1 = 0., q0 = 1., S = a(0) * 1., prodb = b(0) * 1.);
  my(n, v);

  if (all, v = vector(N));
  n = 0;
  while (n < 2*N-1,
    my(q1 = a(n+1) * q0 + b(n) * qm1); qm1 = q0; q0 = q1; n++;
    my(A = a(n+1), B = b(n));
    q1 = A * q0 + B * qm1;
    S += prodb * A / (q1 * qm1);
    if (all, v[(n+1)/2] = S);
    qm1 = q0; q0 = q1; prodb *= b(n+1) * B; n++);
  return (if (all, v, S));
}
