/* Compute S(n) in a stable manner. Output [S(1), ..., S(N)]
 * if 'all' is set, otherwise only S(N). Early abort if S(n)
 * stabilizes for n < N. */
CFforwardstable(a, b, N, all = 0) =
{ my(E = - getlocalbitprec(), e = 2.^(7+E), v, h, C, D);

  if (all, v = vector(N));
  h = a(0) * 1.; if (exponent(h) < E, h = e);
  D = 0.; C = h;
  for (n = 1, N,
    my(A = a(n), B = b(n-1));
    D = A + B * D; if (exponent(D) < E, D = e);
    C = A + B / C; if (exponent(C) < E, C = e);
    D = 1 / D; my(z = D * C);
    h *= z; if (all, v[n] = h);
    if (exponent(z - 1) < E, return (if (all, v[1..n], h))));
  return (if (all, v, h));
}
