/* Return exact result if possible, else floating point */
SQRT(x) = my(a); if (issquare(x, &a), a, sqrt(x));

/* Give the convergence type of a continued fraction as above.
 * fa and fb are rational functions of the variable 'n.
 * Except in certain cases, the coefficients of fa and fb must
 * be numbers, not other variables. */
CFerr() = error("cannot conclude, probably diverges");
CFtype(fa, fb) =
{ my([al, a0, a1, a2] = CFtype_i(fa));
  my([be, b0, b1, b2] = CFtype_i(fb), c0, r1, D, D2, D3, t0);

  r1 = 2*al - be;
  if (b0 <= 0 && r1 <= -1, CFerr());
  if (a0 < 0, a0 = -a0);
  t0 = a0^2 / b0;
  if (r1 > 1,  return ([[al, a0, 0, a1],
                        [r1, -t0, 0, al+2*a1-b1], 1]),
      r1 == 1, return ([[al, a0, 0, a1 + 1/t0],
                        [1,-t0, 0, 2/t0 + al + 2*a1 - b1], 2]),
      r1 ==-1, return ([[be/2, SQRT(b0), t0, b1/2-be/4],
                        [0, -1, 4*t0, 0], 3]),
      r1 ==-2, my(s0 = SQRT(t0));
               return ([[be/2, a0/s0, 0, (b1+s0)/2-be/4],
                        [0, -1, 0, s0], 4]),
      r1 < -2, return ([[-r1/2, SQRT(1/t0), 0, b1/2-a1-be/4],
                        [0, 1, 0, r1/2+1], 5]));
  /* Here r1 = 0: be = 2al */
  D = a0^2 + 4*b0;
  if (D < 0, CFerr());
  if (D > 0,
    my(s = SQRT(D));
    c0 = (a0+s)/2;
    return ([[al, c0, 0, a1 + (b1/2-a1-al/2)*(1-a0/s)],
             [0, -c0^2/b0, 0, (2*a1-b1+al)*a0/s], 6]));
  /* Here be = 2al and a0^2+4b0 = 0 */
  D2 = 2*a1 - b1 + al;
  if (D2 < 0, CFerr());
  if (D2 > 0,
    return ([[al, a0/2, 4*D2, 1/4 + (b1 - al)/2],
             [0, 1, 16*D2, 0], 7]));
  /* Here be = 2al, a0^2 + 4b0 = 0, and 2a1 - b1 + al = 0 */
  D3 = (b1 + al - 1)^2 - be*(al - 1) - 4*(b2 - 2*a2);
  if (D3 <= 0, CFerr());
  my(s3 = SQRT(D3));
  return ([[al, a0/2, 0, a1+(1+s3)/2], [0, 1, 0, s3], 8]);
}

/* auxiliary function for CFtype */
CFtype_i(f) =
{ my(n = 'n, al, a0, a1, a2);

  if (type(f) == "t_CLOSURE", f = f(n));
  al = poldegree(f, n);
  [a0, a1, a2] = Vec(subst(f / n^al, n, 1 / n) + O(n^3));
  return ([al, a0, a1 / a0, a2 / a0]);
}
