/* Compute the M!/m! * w_{m,k} for m <= M; interpolating at
 * 2*N+1 points: (0, +-1, +-2, ..., +- N). */
DiffMinit(M, om) =
{ my(C, h, L, W, Q, N2, N, X, x = 'x);

  N2 = ceil((om+1) * M); N2 += (N2 + 1) % 2; \\ make N2 odd
  N = N2 \ 2;
  Q = x * vecprod(vector(N, k, x^2 - k^2));
  C = binomial(2*N);
  \\ (2N)! / Q'(X[k+1]) = (-1)^(N-k) binomial(2*N, N-k)
  X = vector(N2); L = vector(N2);
  X[1] = 0;
  L[1] = ((Q \ x) + O(x^(M+1))) * (C[N+1] * (-1)^N);
  for (i = 1, N, my(p = 2*i, m = p+1);
    X[p] =  i;
    L[p] = (Q \ (x-i) + O(x^(M+1))) * (C[N-i+1] * (-1)^(N-i));
    X[m] = -i;
    L[m] = subst(L[p], x, -x));
  /* L[k] = (2N)! L_k(x), W[m+1][k] = (2N)!/m! * w_{m,k} */
  W = vector(M+1, m, vector(N2, k, polcoef(L[k], m-1)));
  my(eW = [exponent(w) | w <- W[M+1], w != 0]);
  my(E = vecmax(eW) - vecmin(eW));
  my(B1, B = getlocalbitprec());
  B1 = ceil(B * (1 + 1 / om) + E + 32);
  localbitprec(B1);
  h = 1.0 >> (ceil(B / (N2 - M)));
  return ([X, W, h]);
}

/* [f(a), f'(a), ..., f^(M)(a)/M!]. Optimize assuming that
 * f is computed to B bits in time O(B^om) */
DiffM(f, a = 0, M = 2, om = 2) =
{ my(B = getlocalbitprec(), B1, X, W, Q, h, F, vF);
  [X, W, h] = if (type(om) == "t_VEC", om
                                     , DiffMinit(M, om));
  B1 = bitprecision(h); localbitprec(B1);
  a = bitprecision(a, B1);
  F = [ f(a + h*v) | v <- X ]~;
  Q = powers(1 / h, M, 1 / (#X-1)!);
  \\ Q[m+1] = 1 / (h^m * (2*N)!), vF[m+1] ~ f^(m)(a)/m!
  vF = vector(M+1, m, (W[m] * F) * Q[m]);
  return (bitprecision(vF, B));
}

/* Compute Taylor expansion of f around x = a
 * up to O((x-a))^M */
Taylor(f, a = 0, M = 3, om = 2) = Ser(DiffM(f, a, M - 1, om));
