/* Compute int_a^b f(x) dx using Chebyshev nodes.
 * If given, N must be odd. */
IntChebyshevinit(N = 0) =
{ my(B = getlocalbitprec(), Q, e1, XY, X, Y, W, N2);

  if (!N, N = 2 * ceil(B*log(2)/2) - 1); /* We want N odd. */
  Q = polchebyshev(N);
  localbitprec(B + exponent(Q) + 32);
  e1 = exp(I*Pi / (2*N)); N2 = (N - 1) / 2;
  XY = powers(e1^2, N2, e1); X = real(XY); Y = imag(XY);
  X[N2 + 1] = 0;
  W = vector(N2 + 1, i,
    my(B = Q \ ('x - X[i]));
    (-1)^i * Y[i] * sum(j = 0, N2, my(k = 2*j);
                                   polcoef(B, k) / (k + 1)));
  return (bitprecision([X, W, N], B + 32));
}

IntChebyshev(f, a, b, r = 1, XW = 0) =
{ my(B = getlocalbitprec());
  if (type(XW) == "t_INT", XW = IntChebyshevinit(XW));
  my([X, W, N] = XW, c = b - a, n, S, h, A);
  localbitprec(32); n = ceil(abs(c) * exp(1) / (4 * r));
  localbitprec(B + 32);
  h = c / (2 * n);
  A = vector(n, j, a + h * (2 * j - 1));
  S = SumXW(x -> if (x, my(y = h * x);
                        sum(j = 1, n, f(A[j]+y) + f(A[j]-y))
                      , sum(j = 1, n, f(A[j]), 0.)), XW);
  return (bitprecision(- h * S * (2 / N), B));
}
