/* Given a weight function w and an interval ab = [a, b],
 * perform Gaussian integration. Omit ab if w is already
 * the closure of moments */
IntGaussinit(w, ab = 0, N = 0) =
{ my(B = getlocalbitprec(), mom, NEWB);

  if (type(w) != "t_CLOSURE", return (w));
  localbitprec(32);
  NEWB = 2.25*B; /* or 3*B if Cholesky */
  if (!N, N = ceil(Solvemullog(0, B * log(2))));
  if (ab === 0,
    mom = w
  ,
    my([a,b] = ab);
    mom = (N -> intnum(t = a, b, powers(t, 2 * N - 1, w(t)))));
  while (1,
    localbitprec(NEWB);
    my(P, Q, W, X);
    /* or OrthopolyCholesky(mom(N)) */
    [P, Q] = OrthopolyCF(mom, N);
    X = polrootsreal(P);
    if (poldegree(P) == #X,
      W = subst(Q / P', 'x, X);
      return (bitprecision([X, W], B + 32)));
    /* accuracy loss, increase bitprec by 50%. */
    NEWB = (3*NEWB) >> 1);
}

/* Compute int_a^b w(x)f(x)dx. w can be a closure or the
 * result of IntGaussinit. */
IntGauss(f, a, b, w) =
{ my(XW = IntGaussinit(w, [a, b]));
  SumXW(f, XW);
}
