/* Gaussian integration with weight x^al on ]0, 1]
 * with al > -1 assumed exact. */

IntGaussJacobiinit(al = 0, N = 0) =
{ my(B = getlocalbitprec(), a = al+1, X, W, va, vb, P, Pp, Q);

  if (!al, return (Intgeninit()[1..2]));
  if (!N, N = ceil(B / 4));
  /* This implicitly assumes r / (b - a) >= 1. See text. */
  va = vector(N, k, (a^2+(2*k-3)*a+2*(k-1)^2)
                    / ((a+2*k-2)^2-1));
  vb = vector(N-1, k, k^2*(a+k-1)^2
                      / ((a+2*k-2)*(a+2*k-1)^2*(a+2*k)));
  [P, Q] = Orthopolyrec([va, vb], 1 / a);
  Pp = P';
  localbitprec(B + exponent(Pp)
               - min(0, exponent(subst(Pp, x, 1))));
  X = Orthopolyroots(va, vb, N);
  W = subst(Q / Pp, 'x, X);
  return (bitprecision([X, W], B + 32));
}

IntGaussJacobi(f, a, b, al = 0, XW = 0) =
{ my(B = getlocalbitprec(), c = b - a, S);

  if (type(XW) == "t_INT", XW = IntGaussJacobiinit(al, XW));
  S = c^(al + 1) * SumXW(x -> f(a + c * x), XW);
  return (bitprecision(S, B));
}
