/* int_a^oo x^{al-1} e^{-x} f(x) dx where we assume that
 * f(x) = a_0 + a_1/x^be + a_2/x^(2be) + ... using
 * continued fractions. */
IntGaussLaguerreinvinit(aalbe = [1, 1, 1]) =
{ my([a, al, be] = aalbe, B = getlocalbitprec(), X, W);

  localbitprec(B + 32);
  [X, W] = IntGaussinit(N -> Laguerreinvmoments(aalbe, 2*N));
  X = vector(#X, j, X[j]^(-1 / be));
  return (bitprecision([X, exp(-a)*W], B + 32));
}

IntGaussLaguerreinv(f, aalbe = [1, 1, 1], XW = 0) =
{
  if (!XW, XW = IntGaussLaguerreinvinit(aalbe));
  return (SumXW(f, XW));
}

/* Weight function w(x) = x^{-b al-1}e^{a-1/x^b}$
 * on ]0, 1/a^be]. Given a, al, be, and n >= 0, this
 * computes an (n+1)-component vector containing the
 * moments m_k for 0 <= k <= n for the given scalar product. */
Laguerreinvmoments(aalbe, N) =
{ my([a, al, be] = aalbe, b = 1 / be, ea = exp(a), v, w, ap);

  if (type(b) != "t_INT" || b > N+1, error("incorrect beta"));
  v = vector(N + 1);
  for (k = 0, b - 1, v[k+1] = incgam(al - k * be, a) * ea);
  w = a^(al - 1 + be); ap = a^(-be);
  for (k = b, N,
    w *= ap;
    v[k+1] = if (k * be == al, eint1(a) * ea
                             , (w - v[k+1-b]) / (k*be - al)));
  return (v);
}
