/* Compute int_a^b f(x) dx using Gauss-Legendre integration
 * and interval splitting; assume a and b real. XW is either
 * 0, N, or the output of an IntGaussLegendreinit */
IntGaussLegendresplit(f, a, b, r = 1, XW = 0) =
{ my(B = getlocalbitprec(), bma = b - a, h, S, n, N, X, W);

  if (!bma, return (0.));
  localbitprec(32);
  n = abs(bma) / (4 * r);
  N = if (!XW, ceil(B * log(2) / (2 * max(1, -log(n)))),
          type(XW) == "t_VEC", 2 * #XW[1],
          XW);
  n = ceil(n * 2^(B / (2 * N)));
  localbitprec(B);
  if (type(XW) != "t_VEC", XW = IntGaussLegendreinit(N));
  [X, W] = XW;
  localbitprec(B + 32);
  h = bma / (2*n);
  S = sum(i = 1, #X, W[i] *
          sum(k = 1, n, f(a + (2*k-1 + X[i]) * h) +
                        f(a + (2*k-1 - X[i]) * h)));
  return (bitprecision(h * S, B));
}
