/* Romberg integration using Lagrange interpolation. */

IntLagrangeinit() =
{ my(B = getlocalbitprec(), N, C, D, V);

  N = ceil(LimitmulN(2) * B); if (N%2, N++);
  C = binomial(2*N); \\ C[i+1] = binomial(2N,i)
  D = dirpowers(N, 2*N-1); \\ D[i] = i^(2N-1)
  V = vector(N, m, D[m] * sum(d = 1, N \ m,
                              (-1)^(d*m) * C[N-d*m+1] * D[d]));
  return (V);
}

IntLagrange(f, a, b, V = 0) =
{
  if (type(V) != "t_VEC", V = IntLagrangeinit());
  my(B = getlocalbitprec(), N = #V, D = factorial(2 * N));
  localbitprec(B + Limitaccuracy(2) * N);
  S = sum(m = 2, N, my(h = (b - a) / m);
          V[m] * sum(j = 1, m - 1,
                     if (gcd(j, m) == 1, f(a + j * h))), 0.);
  return ((2*S + V[1] * (f(a) + f(b))) / D);
}
