/* Compute int_a^b f(x) dx by Newton-Cotes methods
 * with equally spaced abscissas. */

IntNewtoninit(N = 0) =
{ my(B = getlocalbitprec(), C, D, T, Q, X, W);

  if (!N, N = 2 * ceil(B * log(2) / 2)); /* We want N even. */
  T = vector(N + 1, j, N * 'x - (j - 1));
  Q = vecprod(T); C = binomial(N);
  W = vector(N + 1, j,
    my(B = subst(intformal(Q \ T[j]), 'x, 1));
    B * (-1)^(N - j + 1) * C[j]);
  D = content(W); W /= D; D /= factorial(N);
  X = vector(#W, i, i);
  return ([X, W, D, N, B + exponent(W) + exponent(D)]);
}

IntNewton(f, a, b, r = 1, XW = 0) =
{ my(c = b - a, n, S, h, A);

  if (type(XW) == "t_INT", XW = IntNewtoninit(XW));
  my(D = XW[3], N = XW[4], Bnew = XW[5]);
  localbitprec(32); n = max(ceil(abs(c) / r), 1);
  localbitprec(Bnew);
  h = c / (N*n);
  A = vector(n, j, a + h * (N * (j-1) - 1));
  S = SumXW(x -> my(y = h * x);
                 sum(j = 1, n, f(A[j] + y), 0.), XW);
  return (c * (D / n) * S);
}
