IntgetN(B, c) =
{ localbitprec(32); B += 64; ceil(B * (log(B) - 5/4) / c); }

/* Precomputations for tanh-sinh numerical
 * integration on a compact interval [a, b]. */
Inta_binit(mul = 1) =
{ my(B = getlocalbitprec(), h, e, ei, X, W, q, r);

  N = IntgetN(B, 6.18 / mul);
  localbitprec(B + 32);
  h = log(2*Pi * N/log(N)) / N;
  X = vector(N); e = exp(h); q = Pi * e;
  W = vector(N); ei = 1/e;   r = Pi * ei;
  for (m = 1, N, \\ q = Pi exp(m h), r = Pi exp(-m h)
    my(c = (q + r)/2, s = q - c, z = 2 / (exp(s) + 1));
    X[m] = 1 - z;
    W[m] = c * z * (2 - z);
    if (exponent(z) < -B - 32,
        X = X[1..m]; W = W[1..m]; break);
    q *= e; r *= ei);
  return ([X, W, h/2]);
}

/* Numerical integration of a closure f on compact [a, b],
 * with precomputed XW = [X, W], otherwise recomputed. */
Inta_b(f, a, b, XW = 0) =
{ my(B = getlocalbitprec(), BX, X, W, W0, c, d, S);

  if (!XW, XW = Inta_binit());
  localbitprec(B + 32);
  [X, W, W0] = XW;
  BX = bitprecision(X[#X]);
  a = bitprecision(a, BX);
  b = bitprecision(b, BX);
  c = (b + a) / 2;
  d = (b - a) / 2;
  S = Pi * f(c) + sum(m = 1, #X,
                      W[m] * (f(c+d*X[m]) + f(c-d*X[m])));
  bitprecision(S * W0 * d, B);
}
