/* [x^a, x^(a+1), ..., x^(a+b)]; b >= 0 */
Powersab(x, a, b) = powers(x, b - 1, x^a);

/* Precomputations for numerical integration on ]-oo, oo[. */
Intoo_ooinit(fast = 0, mul = 1) =
{ my(B = getlocalbitprec(), N, h, e);
  my(vexp, vexpi, vsinh, vcosh, X, W);

  N = IntgetN(B, if (fast, 3.32, 2.213) / mul);
  localbitprec(B + 32);
  h = log(2*Pi*N / log(N)) / N;
  e = exp(h);
  vexp = Powersab(e, 1, N);
  vexpi = Powersab(1 / e, 1, N);
  vsinh = (vexp - vexpi) / 2;
  vcosh = (vexp + vexpi) / 2;
  if (fast,
    for (m = 1, N - 1,
      my(b = exponent(vcosh[m]) + B + 1);
      if (vsinh[m] > 0.347 * b + 1,
          vsinh = vsinh[1..m]; vcosh = vcosh[1..m]; break));
    return (bitprecision([h, vsinh, vcosh], B + 32)));
  vexp = vector(N, m, exp(vsinh[m]));
  vexpi = vector(N, m, 1 / vexp[m]);
  X = (vexp - vexpi) / 2;
  W = (vexp + vexpi) / 2;
  W = vector(N, m, W[m] * vcosh[m]);
  for (m = 1, N - 1,
    if (exponent(W[m]) - 2 * exponent(X[m]) < -B - 64,
        X = X[1..m]; W = W[1..m]; break));
  return (bitprecision([X, W, h], B + 32));
}

/* Numerical integration of a closure f on ]-oo, oo[, with
 * precomputed XW = [h, vabs, vwt], otherwise recomputed. */
Intoo_oo(f, fast = 0, XW = 0) =
{
  if (!XW, XW = Intoo_ooinit(fast));
  my(B = getlocalbitprec(), h = XW[3], S);
  S = f(0) + SumXW(x -> f(x) + f(-x), XW);
  return (bitprecision(S * h, B));
}
