/* Integral from a to infinity of F(x) = f(x)g(x), where f(x)
 * tends to 0 smoothly and slowly at infinity and g is a
 * periodic function of period T, given to larger accuracy
 * than default. */

IntoscLagrange(F, T, alf = [1, 0], XW = 0) =
{ my(B = getlocalbitprec(), U);

  localbitprec(2 * B);
  if (!XW, XW = IntGaussLegendreinit());
  U = (n -> IntGaussLegendre(F, (n-1) * T, n * T, XW));
   return (bitprecision(SumLagrange(U, alf), B));
}
alias(Intosc, IntoscLagrange);

/* Use Sumpos instead */

Intoscpos(F, T, XW = 0) =
{ my(B = getlocalbitprec(), U);

  localbitprec(B + 32);
  if (!XW, XW = IntGaussLegendreinit());
  U = (n -> IntGaussLegendre(F, (n - 1) * T, n * T, XW));
  return (Sumpos(U));
}

Intoscdelta(f, g, T, XW = 0) =
{ my(B = getlocalbitprec(), U);

  localbitprec(B + 32) ;
  if (!XW, XW = IntGaussLegendreinit());
  localbitprec(B);
  U = (n -> IntGaussLegendre(x -> f(x + (n - 1) * T) * g(x),
                             0, T, XW));
  return (Sumdelta(U));
}
