/* Computes int_0^\infty f(x)dx, where f is a regularly
 * oscillating function of half-period H such as a
 * trigonometric or Bessel function. H must be given with
 * a much larger accuracy than default, or left at the
 * default H = 1, with f replaced by f(H*x), in which case
 * H need not be given to higher accuracy. */

IntoscSidiinit(safe = 1) =
{ my(B = getlocalbitprec());

  if (safe, B *= 3 / 2);
  localbitprec(B + 32); IntGaussLegendreinit();
}

IntoscSidi(f, H = 1, XW = 0, safe = 1) =
{
  if (!XW, XW = IntoscSidiinit(safe));
  if (H != 1, f = x -> f(H * x));
  H * SumSidi(x -> IntGaussLegendre(f, x - 1, x, XW), safe);
}

/* faster but less robust version, setting safe = 0 */
IntoscaltSidiinit() = IntoscSidiinit(0);
IntoscaltSidi(f, H = 1, XW = 0) = IntoscSidi(f, H, XW, 0);
