/* Integral from 0 to infinity of f(x)g(x) where f(x) tends
 * to 0 smoothly and slowly at infinity and g is a periodic
 * function of period T, given by its Fourier coefficients
 * G in the format output by Fourier, with exact coefficients.
 * Assume all partial Fourier integrals converge. */

Intperiodicsimple(f, G, T, XWC = 0, XWS = 0) =
{ my(B = getlocalbitprec(), b = 15 - B, S, U, vA);

  localbitprec(2 * B); U = 2 * Pi / T; localbitprec(B + 32);
  my([a0, vC, vS] = G);
  S = if (exponent(a0) > b, a0 * intnum(t = 0, oo, f(t)));
  if (exponent(vC) > b,
    my(fC = (x -> cos(U * x) * vector(#vC, n, f(x / n))));
    if (!XWC, XWC = IntoscOouraMoriinit(T/2, 1/2));
    vA = IntoscOouraMori(fC, T/2, 1/2, XWC);
    S += sum(n = 1, #vC, vC[n] * vA[n] / n));
  if (exponent(vS) > b,
    my(fS = (x -> sin(U * x) * vector(#vS, n, f(x / n))));
    if (!XWS, XWS = IntoscOouraMoriinit(T/2, 0));
    vA = IntoscOouraMori(fS, T/2, 0, XWS);
    S += sum(n = 1, #vS, vS[n] * vA[n] / n));
  return (bitprecision(S, B));
}

/* Possible wrapper function. gG must be a pair [g, G] with
 * G as above or the max nonzero Fourier coefficient. */
Intperiodic(f, gG, T, XWC = 0, XWS = 0) =
{ my([g, G] = gG);

  if (type(G) == "t_INT",
    localbitprec(2 * getlocalbitprec());
    G = Fourier(g, T, G));
  return (Intperiodicsimple(f, G, T, XWC, XWS));
}
