/* Use Taylor expansion at infinity instead of partial
 * fractions. We assume b > a > R. */
IntratTaylor_oo(F, a, b, R) =
{ my(B = getlocalbitprec(), X = variable(F));
  my(L, ser, v, Sa, Sb);

  if ((b == oo || a == -oo) && poldegree(F) >= -1,
    error("diverging integral"));
  localbitprec(32); L = ceil(log(2)*B / log(a/R)) + 1;
  localbitprec(B);
  ser = 1. * subst(F, X, 1/X) + O(X^(L+2));
  v = vector(L, k, polcoef(ser, k+1) / k);
  Sa = Sb = 0;
  forstep (k = L, 1, -1, Sa = (v[k] + Sa) / a);
  if (b != oo, forstep (k = L, 1, -1, Sb = (v[k] + Sb) / b));
  return (Sa - Sb);
}

/* Use c such that max(|a-c|, |b-c|) < min(|al-c|). */
IntratTaylorc(F, a, b, c = "unset") =
{ my(B = getlocalbitprec(), X = variable(F));
  my(L, ser, Sa, Sb, m, r, D = denominator(F)); \\ deg D > 0

  if (c == "unset", c = (a + b) / 2);
  localbitprec(32);
  m = vecmin([norm(z - c) | z <- polroots(D)]);
  a -= c;
  b -= c; r = sqrt(m / max(norm(a), norm(b)));
  if (r <= 1.1, error("Intrat fails"));
  L = ceil(log(2) * B / log(r));
  localbitprec(B);
  ser = intformal(1. * subst(F, X, X + c) + O(X^L));
  [Sa, Sb] = subst(truncate(ser), X, [a, b]);
  return (Sb - Sa);
}
