/* Integration of rational function, assuming that denominator
 * is exact or squarefree, using Ratdec. */

Badpole(z, a, b) =
{ my(B = getlocalbitprec(), s = real(z), t = imag(z));
  return (exponent(t) <= -B && s >= a && s <= b);
}

/* Integral from a to b, finite. Assume a < b */
Intratdeca_b(F, a, b) =
{ my(X = variable(F), DF = denominator(F), NF = numerator(F));
  my(q, S, Poles, co);

  [q, NF] = divrem(NF, DF);
  my([Qa, Qb] = subst(intformal(q), X, [a,b]));
  S = Qb - Qa;
  Poles = Ratdec(NF / DF);
  for (j = 1, #Poles,
    my([z, v, rr, sz] = Poles[j]);
    my(T, P = powers(X - z, v - 1)); \\ P[i+1] = (X - z)^i
    if (Badpole(z, a, b), error("diverging integral"));
    T = sum(k = 0, v-2, polcoef(sz, k) * P[k+1] / (k+1-v));
    if (rr == 2, T = 2 * real(T * conj(P[v])) / norm(P[v])
               , T /= P[v]);
    [Qa, Qb] = subst(T, X, [a,b]);
    S += Qb - Qa;
    co = polcoef(sz, v - 1); if (!co, next);
    co *= log((b - z) / (a - z));
    S += if (rr == 2, 2*real(co), co));
  return (S);
}

/* Integral from a to +oo. */
Intratdeca_oo(F, a) =
{ my(S = 0, Poles = Ratdec(F), X = variable(F), co);

  if (poldegree(F) >= -1, error("diverging integral"));
  for (j = 1, #Poles,
    my([z, v, rr, sz] = Poles[j]);
    my(T, P = powers(X - z, v-1)); \\ P[i+1] = (X - z)^i
    if (Badpole(z, a, oo), error("diverging integral"));
    T = sum(k = 0, v-2, polcoef(sz,k) * P[k+1] / (k+1-v));
    if (rr == 2, T = 2 * real(T * conj(P[v])) / norm(P[v])
               , T /= P[v]);
    S -= subst(T, X, a);
    co = polcoef(sz, v-1); if (!co, next);
    co *= log(a - z);
    S -= if (rr == 2, 2 * real(co), co));
  return (S);
}

/* Integral from -oo to +oo. */
Intratdecoo_oo(F) =
{ my(S = 0, Poles = Ratdec(F), co);

  if (poldegree(F) >= -1, error("diverging integral"));
  for (j = 1, #Poles,
    my([z, v, rr, sz] = Poles[j]);
    if (Badpole(z, -oo, oo), error("diverging integral"));
    co = polcoef(sz, v - 1); if (!co, next);
    co *= I * sign(imag(z));
    S += if (rr == 2, 2 * real(co), co));
  return (S * Pi);
}

/* Driver routine when one can use partial fractions.
Here -oo <= a < b <= oo and a == -oo only if b == oo. */
Intratdec(F, a, b) =
{
  if (a == -oo, return (Intratdecoo_oo(F)));
  if (b ==  oo, return (Intratdeca_oo(F, a)));
  return (Intratdeca_b(F, a, b));
}
