/* If !last, Newton step z - P_N(z) / P'_N(z),
 * else [z - P_N(z) / P'_N(z), (N-1)!P_{N-1}(z)]. */
Legendrenext(N, z, last) =
{ my(q, Z, lz = #z, z2 = apply(x -> x^2, z), n0 = N % 2, a, b);

  if (n0, b = vector(lz, i, 1); a = z2, a = z; b = 0);
  forstep (n = n0, N - 1, 2,
    b = (2 * n + 1) * a - n^2 * b;
    a = (2 * n + 3) * vector(lz, i, z2[i] * b[i])
                    - (n + 1)^2 * a);
  q = vector(lz, i, a[i] / (a[i] - N * b[i]));
  Z = vector(lz, i, z[i] - (z2[i] - 1) * q[i] / (N * z[i]));
  if (!last, return (Z));
  return ([Z, vector(lz, i, b[i] * (1 + q[i]))]);
}

/* initial approximation for positive roots of P_N */
Legendreroots0(N) =
{ localbitprec(32);
  my(z = exp(I * Pi / (4 * N + 2)), z3 = z^3);
  return ((1 - (N-1) / (8 * N^3))
          * real(powers(z3 * z, (N-2) \ 2, z3)));
}

/* positive roots z_i of P_N and (N-1)!P_{N-1}(z_i). */
Legendreroots(N) =
{ my(z = Legendreroots0(N), e = -exponent(1 - z[1]^2));
  my(B = getlocalbitprec(), n = exponent(B + 32 - e) - 2);
  my(pr = 1 + e + (B - e) >> n);

  for (j = 1, n,
    pr = 2 * pr - e;
    z = Legendrenext(N, bitprecision(z, pr), j == n));
  return (bitprecision(z, B));
}
