/* Call lfun init for domain containing s */
Lfuninitcheck(Ldata, s)=
{
  if (type(s) == "t_SER", s = polcoef(s, 0));
  my(re = abs(real(s)), im = abs(imag(s)));
  return (Lfuninit(Ldata, im, re));
}

/* If flag is 1, return Lambda(s), otherwise L(s);
 * if s is a t_SER, return Taylor expansion */
Lfun(Ldata, s, flag = 0)=
{ my(S, [Ldataf, vv] = Lfuninitcheck(Ldata, s));
  my([Vga, k, N, w, R] = Ldataf[3..7], [h, pol] = vv);

  /* First compute Lambda */
  localbitprec( bitprecision(h) );
  if (R && s == k, S = R / 'x + O(1), /* pole at k */
      R && s == 0, S = -w * conj(R) / 'x + O(1), /* pole at 0 */
    /* else treat as regular value */
    my(t, t2, z = exp(h * (s - k/2)));
    t = subst(pol, 'x, z);
    t2 = if (real(s) == k/2, t, subst(pol, 'x, conj(1/z)));
    if (R, t += R * (1 / expm1(h * (s - k)) + 1/2);
           t2+= R * (1 / expm1(-h * conj(s)) + 1/2));
    S = h * (t + w * conj(t2)));
  /* Divide by factor at infinity to get L */
  if (!flag, S /= N^(s/2) * Gammafactor(Vga, s));
  if (type(s) != "t_SER" && valuation(S, 'x) >= 0,
      S = polcoef(S, 0));
  return (S);
}

/* Pi^(-s/2) gamma(s/2) */
GammaR(s) =
{
  s /= 2;
  if (type(s) == "t_INT" && s <= 0, s += 'x / 2 + O('x^2));
  Pi^(-s) * gamma(s);
}

/* Gamma factor at s. */
Gammafactor(Vga, s)= vecprod([GammaR(s + a) | a <- Vga]);
