/* CHI is a primitive Dirichlet character given either by an
 * integer D representing the Kronecker symbol (D/.), or by a
 * vector [CHI[1],...,CHI[N]]. */
Lfunchicreate(CHI) =
{ my(t = type(CHI));
  my(triv = CHI == 1 || (t == "t_VEC" && #CHI == 1));

  if (triv,         Lfunzetacreate(), /* trivial character */
      t == "t_INT", Lfunchiquadcreate(CHI), /* quadratic */
                    Lfunchiveccreate(CHI));
}

/* Riemann zeta function */
Lfunzetacreate() =
  [n -> vector(n, j, 1), 0, [0], 1, 1, 1, 1];

/* Quadratic character (D/.) */
Lfunchiquadcreate(D) =
{ my(an, L);
  an = (n -> vector(n, j, kronecker(D, j)));
  L = [an, 0, [D < 0], 1, abs(D), oo, 0];
  L[6] = Lfunrootres(L)[2]; return (L);
}

/* General case chi(i) = v[i] for i = 1..N */
Lfunchiveccreate(v) =
{ my(N = #v, odd, sd, an);
  odd = exponent(v[N - 1] - 1) > -32; /* is chi odd ? */
  sd = exponent(imag(v)) < -32; /* is chi self-dual ? */
  an = (n -> vector(n, j, v[(j - 1) % N + 1]));
  L = [an, 1 - sd, [odd], 1, N, oo, 0];
  L[6] = Lfunrootres(L)[2]; return (L);
}
