/* Initialization: returns a vector [Ldataf,VV], where Ldataf
 * (f for full) is the initial Ldata component structure with
 * the root number and/or residues computed if not given
 * (i.e., set to oo), and VV is a technical vector. Allows to
 * compute L(s) for |Re(s)| <= maxs and |Im(s)| <= maxt. */
Lfuninit(Ldata, maxt = 50, maxs = 10)=
{ my(B = getlocalbitprec(), h, M, L, tmin, E, sig, m0);

  localbitprec(64);
  my(NEWB = ceil(B + Pi/4 / log(2) * maxt * Ldata.d));
  if (#Ldata == 2,
    my([t, s] = Ldata[2][4..5]);
    if (t >= maxt && s >= maxs
                  && Ldata.prec >= NEWB, return (Ldata));
    Ldata = Ldata[1]); /* recompute Lfuninit */
  my([Vga, k, N, w, R] = Ldata[3..7], d = #Vga);
  maxt = max(abs(maxt), 2);
  sig = vecmax([1, maxs, k - maxs]);
  E = log(2) * NEWB;

  localbitprec(NEWB); tmin = 1;
  if (w == oo || R == oo || bitprecision([R, w]) < NEWB,
    tmin =  1 / sqrt(2));  /* need to (re)compute w or R */
  my(vv, [an, K] = vv = Lfunthetainit(Ldata, tmin));
  L = #an;
  if (tmin != 1,
    [R, w] = Lfunrootres(Ldata, vv);
    L = ceil(L * tmin));
  h = max(Pi * maxt * d/2, (d*(sig-1) - 1) * log(E));
  h = d * Pi^2 / (2 * (E + h));
  m0 = ceil(log(2) / h); h = log(2) / m0;
  my(Ap = sig + (1 - d + vecsum(Vga)) / d);
  my(Ep = (E + (sig / 2) * log(N)) / (d * Pi));
  M = log(N) / 2 + (d / 2) * log(Ep + (Ap/(2*Pi))*log(Ep));
  M = ceil(M / h) + 5;
  my(isqN = 1 / sqrt(N), LL = log(L), v0 = floor(LL / log(2)));
  my(e = exp(h), ek = powers(e^(k/2), M)); ek[1] = 1/2;
  my(V = matrix(m0, v0+1), eh = powers(e, m0-1));
  my(upow = dirpowers(L+1, 2 / d));
  for (i = 0, m0 - 1,
    for (j = 0, v0,
      my(t = eh[i+1] << j);         /* exp(i*h)*2^j */
      my(U = (floor(L / t) - 1) \ 2);
      my(C = (t * isqN)^(2 / d));
      V[i+1, j+1] = sum(u = 0, U,
        my(v = vector(j+1, a, an[(2*u+1) << (a-1)]));
        if (v, v *= K(C * upow[2*u+1])))));
  my(c = vector(M+1));
  for (m = 0, M,
    my([q, r] = divrem(m, m0), v = (LL - r*h) / log(2));
    c[m+1] = sum(Q = q, v, V[r+1, Q+1][Q-q+1]));
  my(pol = Polrev(vector(M+1, m, ek[m] * c[m])));
  Ldata[6] = w;
  Ldata[7] = R; return ([Ldata, [h, pol, R, maxt, maxs]]);
}
