/* Round the root number if close to 1, -1, I or -I. */
Roundw(w)=
{ my(e, W = round(w, &e), B = getlocalbitprec());
  return (if (e < -B/2, W, w));
}

/* Compute the residue R of Lambda at k and the root number.
 * Returns [R, w]. If vv is set, keep the current accuracy,
 * otherwise increase it to perform the computation. */

/* assume R = 0 */
Lfunrootresnopole(Ldata, vv, B) =
{ my(K1 = Lfuntheta(Ldata, 1, vv), K2);

  if (exponent(K1) > -B / 2,
    K2 = K1
  ,
    [K1, K2] = Lfunthetaroot2(Ldata, vv);
    K2 *= 2^(-Ldata.k / 2));
  bitprecision([0, Roundw(K2 / conj(K1))], B);
}

/* [A,B,C] such that A * w + B * R = C from
 * K1 = theta(t) and K2 = theta(1/t), T = t^(2k) */
LfunthetaABC(K1, K2, T) =
  [ conj(T * K1 - K2), T - 1, T^(1/2) * (K2 - K1) ];

Lfunrootres(Ldata, vv = 0) =
{ my(K1, K2, B = getlocalbitprec());

  if (#Ldata == 2, Ldata = Ldata[1]);
  localbitprec(if (!vv, B + 32, B));
  if (!vv, vv = Lfunthetainit(Ldata));
  my([sd, Vga, k, N, w, R] = Ldata[2..7]);
  if (!R, return (Lfunrootresnopole(Ldata, vv, B)));
  if (sd == 0 && w == -1,
    R = Lfuntheta(Ldata, 1, vv);
    return (bitprecision([R, w], B)));
  [K1, K2] = Lfunthetaroot2(Ldata, vv);
  my([a, b, c] = LfunthetaABC(K1, K2, 2^k));
  if (w != oo && precision(w) == oo,
    R = (c - a * w) / b;
    return (bitprecision([R, w], B)));
  my(t0 = 11/10);
  K1 = Lfuntheta(Ldata, t0, vv);
  K2 = Lfuntheta(Ldata, 1 / t0, vv);
  my([a2, b2, c2] = LfunthetaABC(K1, K2, t0^(2*k)));
  [w, R] = [a, b; a2, b2]^(-1) * [c, c2]~;
  return (bitprecision([R, Roundw(w)], B));
}
