/* Initialization for Theta computations, |t| >= tmin. */
Lfunthetainit(Ldata, tmin = 1, Nmax = Ldata.N)=
{ my(B = getlocalbitprec(), Vga = Ldata.Vga, d = #Vga, A, E, L);
  A = (1 - d + vecsum(Vga)) / d;
  E = log(2) * B / (d * Pi);
  L = ceil((E + A/(2*Pi) * log(E))^(d/2) * sqrt(Nmax) / tmin);
  return ([Ldata.a(L + 5), Mellininvtech(Vga)]);
}

/* Theta0(t) = sum_{n > 0} a(n)K(nt). Treat specially the easy
 * but important cases where #Vga = 1 or Vga = [0,1] in which
 * case K(t) is an exponential. */
Lfuntheta0(Ldata, t, vv = 0)=
{ my(v, q, q1, q2, S);

  if (!vv, vv = Lfunthetainit(Ldata));
  my([an, K] = vv, T = min(#an, ceil(#an / real(t))));
  my(Vga = Ldata.Vga, d = #Vga);
  if (d == 1,
    v = vector(T); q1 = q = exp(-Pi * t^2); q2 = q * q;
    v[1] = q;
    for (n = 2, T, q1 *= q2; q *= q1; v[n] = q);
    return (2*sum(n = 1, T, an[n] * (n*t)^Vga[1] * v[n])));
  if (d == 2 && (Vga = vecsort(Vga)) == [0,1],
    q = exp(-2 * Pi * t); S = 0;
    forstep (n = T, 1, -1, S = an[n] + q * S);
    return (2 * q * S));
  t = t^(2 / d); v = dirpowers(T, 2 / d);
  S = sum(n = 1, T, if (an[n], an[n] * K(v[n] * t)));
  return (S);
}

Lfuntheta(Ldata, t, vv = 0)=
  Lfuntheta0(Ldata, t / sqrt(Ldata.N), vv);

/* Accessors to retrieve information from Ldata.
 * Also support Lfuninit format [Ldataf, VV] */
Ldata.a    = if (#Ldata == 2, Ldata[1][1], Ldata[1]);
Ldata.Vga  = if (#Ldata == 2, Ldata[1][3], Ldata[3]);
Ldata.k    = if (#Ldata == 2, Ldata[1][4], Ldata[4]);
Ldata.N    = if (#Ldata == 2, Ldata[1][5], Ldata[5]);
Ldata.w    = if (#Ldata == 2, Ldata[1][6], Ldata[6]);
Ldata.R    = if (#Ldata == 2, Ldata[1][7], Ldata[7]);
Ldata.prec = bitprecision(Ldata[2][1]);
Ldata.d    = #Ldata.Vga;
/* residue of L at s = k */
Ldata.r = my(k = Ldata.k); \
  Ldata.R / (Ldata.N^(k/2) * Gammafactor(Ldata.Vga,k));
