/* Search and store zeros of Hardy Z-function on critical line,
 * Im(z) in range = [T,Tmax]. n0 = order of 0 on real axis if
 * T == 0 (else n0 = 0). For each t, assume there is at most a
 * single zero in [t, t + Gap(t)] */
Zerosearch(Z, Gap, n0, range) =
{ my([T, Tmax] = range, T0);
  my(s = sign(Z(T)), w = List(vector(n0)));

  if (n0, T = 2.^(-getlocalbitprec()/ (2*n0)));
  T0 = T;
  while (T < Tmax,
    T = min(T + Gap(T), Tmax);
    if (s == sign(Z(T)), next);
    listput(~w, solve(t = T0, T, Z(t)));
    s = -s; T0 = T);
  return (Vec(w));
}

/* For an L function of degree d and conductor N, the expected
 * gap between two consecutive critical zeros at height t and
 * t2 is 2*Pi / e(t). Return 2*Pi / (divz*e(t)) as a closure */
Zerogap(N, d, divz) =
{ my(C = d + 2 * log(max(1, N / (Pi/2)^d)));
  t -> my(e = d * log(max(1, t / (2*Pi))) + C);
       2 * Pi / (divz * e);
}

/* Naive search for zeros on the critical line for t in
 * range = [a, b]. Mesh size is controlled by divz, increase
 * if necessary. */
Lfunzeros(Ldata, range = [0,50], divz = 8) =
{ my(Z, n0, Gap);

  Ldata = Lfuninit(Ldata, vecmax(abs(range)), Ldata.k / 2);
  my(NEWB = Ldata.prec);
  Gap = Zerogap(Ldata.N, Ldata.d, divz);
  n0 = if (range[1] == 0, Lfunorderzero(Ldata)[1]);
  Z = Zerosearch(t -> LfunHardy(Ldata, bitprecision(t,NEWB)),
                 Gap, n0, range);
  return (bitprecision(Z, getlocalbitprec()));
}
