/* 'pol' is an irreducible monic polynomial in Z[X] defining a
 * number field K. Creates the L function zeta_K(s) */
Lfunzetakcreate(pol) =
{ my(d = poldegree(pol), K = bnfinit(pol), h = K.no, Vga, R);
  my([r1, r2] = K.sign, N = abs(K.disc), w = K.tu[1]);

  Vga = concat(vector(d - r2), vector(r2, j, 1));
  R = Gammafactor(Vga, 1) * h * K.reg * 2^r1 * (2 * Pi)^r2 / w;
  return ([(n -> dirzetak(K, n)), 0, Vga, 1, N, 1, R]);
}

/* Creates zeta_K(s) / zeta(s): reduces the degree and cancels
 * the pole */
Lfunzetakcreate2(pol) =
{ my(d = poldegree(pol) - 1, K = nfinit(pol));
  my([r1, r2] = K.sign, N = abs(K.disc), Vga, r, an);

  Vga = concat(vector(d - r2), vector(r2, j, 1));
  an = (n -> dirdiv(dirzetak(K, n), vector(n, j, 1)));
  return ([an, 0, Vga, 1, N, 1, 0]);
}

