Limitalf(alf) =
{ if (type(alf) == "t_VEC",     alf,
      type(alf) == "t_CLOSURE", [1, alf],
      [alf, 0]); }

/* Compute lim_{n -> oo} u(n) using Lagrange interpolation,
 * assuming u(n) = a_0 + f(n) * sum_{m>=1} a_m / n^{al m},
 * alf is the scalar al or a closure f (al = 1) or the pair
 * [al, f] */
Limitinit(alf = 1) =
{ my(B = getlocalbitprec(), N, B1, W, T);
  my([al, f] = Limitalf(alf));

  N = ceil(LimitmulN(al) * B);
  B1 = ceil(B + Limitaccuracy(al) * N);
  localbitprec(B1); al = bitprecision(al, B1);
  W = Limitweights(N, al);
  if (!f, return ([B1, W, 1]));
  W = vector(#W, n, W[n] / f(n));
  T = vecsum(W); if (exponent(T) < -B/2, error("T ~ 0"));
  return ([B1, W, vecsum(W)]);
}

/* vu(N) = [u_1, ..., u_N] */
Limitvec(vu, alf = 1, tab = 0) =
{
  if (!tab, tab = Limitinit(alf));
  my(U, B = getlocalbitprec(), [B1, W, T] = tab, S);
  localbitprec(B1); U = vu(#W);
  S = sum(n = 1, #W, W[n] * U[n], 0.);
  return (bitprecision(S / T, B));
}

/* u(n) = u_n */
Limit(u, alf = 1, tab = 0) =
  Limitvec(N -> vector(N, n, u(n)), alf, tab);
