/* Given a sequence u(n) with asymptotic expansion
 * u(n) = a0 + a1 / n^be + a2 / n^{al+be} + a3 / n^{2al+be} ...
 * with al,be > 0; find a0 by Richardson extrapolation. */
LimitRichardsoninit(alf = [1, 0]) =
{ my(B = getlocalbitprec(), al, be, n, k, T, e, ex2, v);

  if (type(alf) == "t_VEC",
    [al, be] = alf; if (!be, be = al)
  ,
    al = be = alf);
  localbitprec(32);
  localbitprec(sqrt(B / al));
  n = ceil(2^sqrt(B / al));
  k = logint(n, 2);
  localbitprec(B + 32);
  e = 2.0^(be - al); ex2 = 2^al; T = (-1)^(k - 1);
  v = vector(k - 1, j, e *= ex2; T *= e - 1; 'x - e);
  return ([n, k, T, vecprod(v)]);
}

LimitRichardson(u, alf = [1, 0], tab = 0) =
{ my(B = getlocalbitprec(), S);

  if (!tab, tab = LimitRichardsoninit(alf));
  my([n, k, T, pol] = tab);
  localbitprec(B + 32);
  S = sum(j = 0, k - 1, polcoef(pol, j) * u(2^(k-j-1)*n));
  return (bitprecision(S / T, B));
}
