/* Given a symmetric positive definite matrix A, finds an
 * upper triangular matrix R such that R~ * R = A. */
MatCholesky(A) =
{ my([m, n] = matsize(A), R = matrix(n, n));

  if (n != m, error("A not square"));
  for (i = 1, n,
    R[i,i] = sqrt(A[i,i] - sum(k = 1, i - 1, R[k,i]^2));
    my(c = 1 / R[i,i]);
    for (j = i + 1, n,
      R[i,j] = c * (A[i,j]
                    - sum(k = 1, i - 1, R[k,i] * R[k,j]))));
  return (R);
}
