/* Vector of coefficients M_n, n <= N; if asymptotic expansion
 * is finite, return a possibly shorter polynomial */
Mellininvlargecoefs(Vga, N) =
{ my(d = #Vga, dv = 0, x = 'x, pol, S, A, sh, vp, M);

  if (d == 1, return (Pol(1)));
  if (d == 2 && (dv = abs(Vga[1] - Vga[2])) % 2,
    N = min(N, (dv+1) \ 2));  /* finite expansion */
  pol = prod(i = 1, d, 1 + d * Vga[i] * x);
  A = - vecsum(Vga);
  S = [sum(k = 0, m, A^k * binomial(k + d - m, k)
                     * polcoef(pol, m - k)) | m <- [0..d]];
  sh = powers(sinh(x + O(x^(d+2))) / x, d);
  vp = vector(d, p,
    my(Sp = [S[m+1] * prod(j = m, p-1, d-j) | m <- [0..d]]);
    my(w = vector(p\2 + 1, k, polcoef(sh[d-p+1], 2*k-2)));
    my(D = -(2 * d)^p);
    vector(N - 1, n,
      my(c, a = 2*n - p + 1);
      my(v = vector(p+1, k, a^(k-1) / (k-1)!));
      sum(m = 0, p,
          if (c = Sp[m+1],
            my(q = p - m);
            c * sum(k = 0, q \ 2, v[q-2*k+1] * w[k+1]))) / D));
  M = vector(N); M[1] = 1;
  for (n = 1, N - 1,
    M[n+1] = sum(p = 2, min(d, n+1), vp[p][n] * M[n+2-p]) / n);
  return (if (dv, Polrev(M), M));
}
