/* Given 2n moments m_k for 0 <= k < 2n stored in
 * M[1]...M[2n], compute the coefficients
 * [a_0, ..., a_{n-1}] and [b_1, ..., b_{n-1}] of the 3-term
 * recursion P_{j+1}(X) = (X - a_j)P_j(X) - b_jP_{j-1}(X);
 * P_1 = X - a_0, P_0 = 1. */

Threeterm(M) =
{ my(n = #M \ 2, N = n + 1, A, R, p, v, a, b);

  /* M[2n+1] is outside of range: replace by 0 */
  A = matrix(N, N, i, j, if (i != j || i != N, M[i+j-1]));
  R = MatCholesky(A);
  p = vector(n, j, 1 / R[j, j]);
  v = vector(n, j, R[j, j+1] * p[j]);
  a = vector(n - 1, j, if (j == 1, M[2]/M[1], v[j] - v[j-1]));
  b = vector(n - 1, j, (R[j+1, j+1] * p[j])^2);
  return ([a, b]);
}

/* Q1 = q1, ab = [a, b], compute [P_n, Q_n]. */
Orthopolyrec(ab, q1) =
{
  my([a, b] = ab, P0 = 1, P1 = 'x - a[1], Q0 = 0, Q1 = q1);
  for (j = 1, #a - 1,
    my(A = 'x - a[j+1]);
    [P0, P1] = [P1, A * P1 - b[j] * P0];
    [Q0, Q1] = [Q1, A * Q1 - b[j] * Q0]);
  return ([P1, Q1]);
}

/* Given 2n moments m_k for 0 <= k < 2n, compute [P_n, Q_n]. */
OrthopolyCholesky(M) = Orthopolyrec(Threeterm(M), M[1]);
