/* Solve x - a * log(x) = b with a > 0 and b >= a * (1-log(a));
 * one solution is x = -a * W_{-1}(-exp(-b / a) / a). */
Solvedivlog(a, b = 0) =
{ localbitprec(32); [a, b] = bitprecision([a, b], 32);
  my(x = a * log(a) + b);
  if (a <= 0 || x < a,
    error("a <= 0 or b too small in Solvedivlog"));
  for (i = 1, 2, x = (a * (log(x) - 1) + b) / (1 - a / x));
  return (x);
}

/* Solve x * log(x) - a * x = b with b > 0;
 * solution is x = b / W_0(b * exp(-a))). */
Solvemullog(a, b) =
{ localbitprec(32); [a, b] = bitprecision([a, b], 32);
  if (b <= 0, error("b <= 0 in Solvemullog"));
  return (b / lambertw(b * exp(-a)));
}

/* Solve a * x - x * log(x) = b; b > 0 and a >= 1 + log(b);
 * one solution is x = -b / W_{-1}(-b * exp(-a)); */
Solvemulneglog(a, b) =
{ localbitprec(32); [a, b] = bitprecision([a, b], 32);
  my(x = b / a, y = a - log(b) - 1, e = exponent(y));
  if (b <= 0 || y < 0,
    error("a < 1 + log(b) in Solvemulneglog"));
  for (i = 1, 5 - e, x = (b - x) / (a - 1 - log(x)));
  return (x);
}
