/* Compute sum_{n >= 1} f(n) under the Abel-Plana assumptions.
 * We assume for simplicity that f tends
 * to 0 slowly at infinity, and has no pole z with Re(z) >= B/3
 * with B number of desired binary digits. */
alias(SumAbelPlanainit, IntGaussexpinit);

/* Set isreal = 1 if f satisfies f(conj(z) = conj(f(z)). */
SumAbelPlana(f, F = 0, isreal = 0, tab = 0) =
{ my(B = getlocalbitprec(), N, S);

  if (type(tab) != "t_VEC", tab = SumAbelPlanainit(tab));
  N = max(ceil(B / 3), 2);
  F = Sumintinit(f, F);
  S = sum(m = 1, N - 1, f(m), f(N) / 2.) - F(N);
  if (isreal,
    S -= IntGaussexp(t -> imag(f(N+I*t)) / t,
                     1, tab)
  , /* else */
    S += I/2*IntGaussexp(t -> (f(N+I*t) - f(N-I*t)) / t,
                         1, tab));
  return (S);
}
