/* Antiderivative of f which vanishes at infinity. F is either
 * already such a primitive or indicates expansion at infinity
 * in the format expected by IntGaussa_ooinit [t_VEC] or
 * Inta_ooinit [a scalar]. */
Sumintinit(f, F = 0) =
{ my(tF = type(F), XW);

  if (tF == "t_CLOSURE",, \\ do nothing
      tF == "t_VEC", XW = IntGaussa_ooinit(F);
	             F = N -> -IntGaussa_oo(f, N, F, XW)
                   , XW = Inta_ooinit(F);
                     F = N -> -Inta_oo(f, N, F, XW));
  return (F);
}

/* Let f be a closure. Compute sum_{m >= 1} f(m)
 * under the Euler-MacLaurin assumptions. Derivatives are
 * computed using power series substitutions. */
SumEulerMaclaurin(f, F = 0) =
{ my(B = getlocalbitprec(), N = 2 * B, k, S, ser);

  F = Sumintinit(f, F);
  k = ceil(0.1235 * B); if (k % 2, k++);

  bernvec(k \ 2); /* initialize Bernoulli cache */
  localbitprec(B + 32);
  ser = f(N + 'x + O('x^(k + 1)));
  S = sum(m = 1, N - 1, f(m), f(N) / 2.) - F(N)
      - sum(j = 1, k \ 2, bernfrac(2*j)/(2*j)
                          * polcoef(ser, 2*j-1), 0.);
  return (bitprecision(S, B));
}
