/* Compute sum_{n >= 1} u(n) using Lagrange extrapolation.
 * Assume that sum_{j > n} u(j) has a regular asymptotic
 * expansion f(n)*sum_{i >= 0} a_i/n^(al*i). alf is the scalar
 * al or a closure f (al = 1) or the pair [al, f].
 * Increase mul to improve accuracy */

/* vu(N) = [u(1), ..., u(N)]  */
SumLagrangevec(vu, alf = 1, mul = 1, tab = 0) =
{
  Limitvec(N -> my(S = 0., U = vu(N * mul));
    vector(N, j, S += sum(n = 1 + (j - 1) * mul,
	   j * mul, U[n])), alf, tab);
}
SumLagrange(u, alf = 1, mul = 1, tab = 0) =
  SumLagrangevec(N -> vector(N, n, u(n)), alf, mul, tab);
