/* c = al, [al, be], [al, n->w(n)] or [al, [n->w(n), fast]]
 * Return [al, be, n -> w(n)] */
Getalbew(c) =
{
  if (type(c) == "t_CLOSURE", return ([1, 0, c]));
  if (type(c) != "t_VEC", return ([c, 0, 0]));
  my([a, b] = c);
  if (type(a) == "t_CLOSURE", return ([1, -b, a]));
  if (type(b) == "t_CLOSURE", return ([a, 0, b]));
  if (type(b) != "t_VEC", return ([a, b, 0]));
  my([w, fast] = b); return ([a, -fast, w]);
}

/* Initialize weights and abscissas for generalized Monien
 * summation sum_{n >= 1} f(n) where
 * f(n) = w(n) * sum_{m >= 0} a_m / n^{al m},  al > 0,
 * w(n) = O(n^{fast + epsilon}); fast must be set correctly. */
SumMonieninit(alff = 1, N = 0) =
{ my(B = getlocalbitprec(), BN, v, vr, vzeta, PQ, Q);
  my(X, W, [al, be, w] = Getalbew(alff));

  if (al <= 0, error("divergent SumMonieninit"));
  if (!w && be == 0, be = al * (1 + floor(1 / al)));
  if (!w && al == 2 && be == 2, return (SumMonien2init(N)));
  BN = ceil(B / al);
  localbitprec(32);
  my(c1 = 1 - log(2), c2 = log(2) / 2);
  N = max(N, ceil(Solvemullog(c1, c2 * BN))) + 1;
  my(NEWB = ceil(1.6 * max(1, al) * BN));
  my(jin = floor((1 - be) / al) + 1);
  while (1,
    localbitprec(NEWB);
    al = bitprecision(al, NEWB);
    vzeta = if (!w,
      -zeta(vector(2 * N, m, al * (m - 1) + be))
    , /* else */
      Sumdelta(m -> my(M = m^(-al));
                    -w(m) * powers(M, 2 * N - 1, M^jin),
               -be - jin * al));
    v = Quodif(vzeta);
    PQ = CFback(2 * N, n -> v[n+1] * 'x);
    Q = denominator(PQ);
    my(int = if (!w && type(al) == "t_INT", al, 0));
    vr = SumMonienroots(Q, int, polrootsreal);
    if (poldegree(Q) == #vr, break);
    /* here accuracy loss, increase it by 50%. */
    NEWB = (3*NEWB) >> 1);
  X = vector(N, j, vr[j]^(1/al));
  W = subst(numerator(PQ) / Q', 'x, vr);
  if (!w,
    my(c = be - 2 * al);
    if (c, W = vector(N, j, W[j] * X[j]^c))
  , /* else */
    W = vector(N, j, W[j] * vr[j]^(jin - 2) / w(X[j])));
  return (bitprecision([X, W], B + 32));
}

/* Compute sum_{n >= 1} w(n) f(n). If XW a t_INT, recompute XW,
 * otherwise consider that XW = [X, W]. alff = [al, w, fast]
 * with everything optional. */
SumMonien(f, alff = 1, XW = 0) =
{
  if (type(XW) != "t_VEC", XW = SumMonieninit(alff, XW));
  return (SumXW(f, XW));
}
