/* Compute the constant z(f) occurring in Euler-MacLaurin
 * type formulas (see text), in particular equal to
 * sum_{n >= 1} f(n) - int_1^oo f(t)dt if series and integral
 * converge. The integer k >= 1 is such that f^(k)(x) tends
 * to 0 at infinity. */

SumRamanujanz(f, k = 1) =
{ my(k2 = k \ 2, P = bernpol(k), F = derivn(f, k));
  my(V, S, T);

  V = derivnum(x = 1, f(x), concat(0, vector(k2, j, 2*j-1)));
  S = V[1] / 2
      - sum(j = 1, k2, bernfrac(2*j) / (2*j)! * V[j + 1]);
  T = IntoscSidi(t -> subst(P, 'x, frac(t)) * F(t + 1));
  return (S + (-1)^(k-1) / k! * T);
}

