/* Compute S = sum_{n >= 1} f(n) by EM symmetric
 * Delta_{1/4} summation. */
Sumdeltainit(N = 0) =
{ my(B = getlocalbitprec(), C, k, S, X, W, vS);

  localbitprec(32);
  my(E = B * log(2));
  if (!N,
    my(w = lambertw(4));
    N = ceil(E / (w * (1 + w)) + 5);
    k = N * w;
  ,
    if (4 * N < E, error("N too small in Sumdeltainit"));
    k = Solvemulneglog(1 + log(4 * N), E));
  k = ceil(k); if (k%2, k--);
  localbitprec(B + 32);
  C = vector(k \ 2, j, binomial(2 * j - 1));
  S = 4 * log('x / 2 + sqrt(1.0 + 'x^2 / 4 + O('x^(k+3))));
  S = 1 / expm1(S) - 1 / S;
  k \= 2;
  vS = vector(k, j, (-1)^j * polcoef(S, 2 * j - 1));
  W = vector(k, m, (-1)^m*sum(j = m, k, vS[j] * C[j][j-m+1]));
  X = vector(k, m, (2 * m - 1) / 4);
  return ([X, W, N]);
}

/* F:   fast or closure giving integral of f from N to oo;
 * tab: 0 or vector [X, W, N] output of init. */
Sumdelta(f, F = 0, tab = 0) =
{ my(B = getlocalbitprec(), S);

  if (!tab, tab = Sumdeltainit());
  my(N = tab[3]);
  F = Sumintinit(f, F);
  S = SumXW(x -> f(N - x) - f(N + x), tab);
  localbitprec(B + 32);
  S += sum(m = 1, N - 1, f(m), f(N) / 2.) - F(N);
  return (bitprecision(S, B));
}
