/* Compute sum_{n >= 1} z^n u(n), u being a closure.
 * Since there may be enormous accuracy changes, z should
 * be given as a closure, but scalar accepted. */

/* evaluate a quantity given as a scalar or a function: the
 * evaluation occurs at current dynamic precision */
Getz(z) = return (if (type(z) == "t_CLOSURE", z(), z));

/* Compute weights for Sumoscinit, bit and N come from
 * SumoscinitBN. Used by Sumchi. */
Sumoscinit0(bit, N, z) =
{ my(c, d, powz, zi, bic, vc = vector(N));

  localbitprec(bit);
  z = Getz(z); zi = 1 / z;
  d = (1 - 2 * zi + 2 * sqrt(zi * (zi - 1)))^N;
  powz = powers(z, N);
  d = (d + 1 / d) / 2; bic = -1.; c = d;
  for (j = 0, N - 1,
    c += bic;
    vc[j + 1] = c * powz[j + 1];
    bic *= zi * (j + N) * (j - N) / ((j + 1/2) * (j + 1)));
  return ([d, vc, bit]);
}

/* Bit accuracy and number of terms needed by Sumoscinit.
 * Used by Sumchi */
SumoscinitBN(z) =
{ my(B = getlocalbitprec(), N, d, ct, zi, bit);

  localbitprec(32);
  zi = 1 / Getz(z);
  d = abs(1 - 2 * zi + 2 * sqrt(zi * (zi - 1)));
  ct = abs(log(d)) / log(2);
  if (exponent(ct) < -8, error("non-oscillating function"));
  N = ceil(B / ct);
  bit = ceil(N * (log(3 + sqrt(8)) / log(2))) + 32;
  return ([bit, N]);
}

/* Initialization function for Sumosc */
Sumoscinit(z = -1) =
{ my([bit, N] = SumoscinitBN(z));
  return (Sumoscinit0(bit, N, z));
}

/* Compute sum_{n>=1} z^n * u(n). */
Sumosc(u, z = -1, tab = 0) =
{ my(B = getlocalbitprec(), bitnew, d, vc, S);

  if (!tab, tab = Sumoscinit(z));
  [d, vc, bitnew] = tab;
  localbitprec(bitnew);
  S = sum(j = 1, #vc, vc[j] * u(j));
  return (bitprecision(Getz(z) * S / d, B));
}
