/* P = sorted prime divisors of squarefree n;
 * V[k] = k^s for k <= sq = floor(sqrt(N))
 * Return n^s if P^+(n) <= sq, else return 0 */
Smallfact(n, P, sq, V) =
{
  if (!#P, return (1));
  my(p = P[#P]); if (p > sq, return (0)); /* P^+(n) > sqrt(N) */
  my(c = V[p]);
  forstep (i = #P - 1, 1, -1,
    n /= p; if (n <= sq, return (V[n] * c));
    p = P[i]; c *= V[p]);
  return (c);
}

/* Compute sum_{1 <= n <= N} n^s. */
Sumpow(N, s) =
{
  if (N < 10000, return (vecsum(dirpowers(N, s))));

  my(B = getlocalbitprec(), sq = sqrtint(N));
  my(W = vector(sq), F = vector(sq), V, precp, t, S);
  s = precision(s, B + 32);
  localbitprec(B + 32);
  V = dirpowers(sq, s); W[1] = F[1] = 1;
  for (n = 2, sq,
    W[n] = W[n - 1] + V[n];       /* sum_{j <= n} j^s */
    F[n] = F[n - 1] + sqr(V[n])); /* sum_{j <= n} j^(2s) */
  precp = t = 1; S = 0;
  forprime (p = sq + 1, N,
    t *= (p / precp)^s;
    S += W[N \ p] * t; precp = p);
  /* At this point S = S_1(N). Now add S_2(N). */
  forsquarefree (dF = 1, N,
    my(d = dF[1]);
    if (gcd(d, 6) == 1 && t = Smallfact(d, dF[2][,1], sq, V),
      my(q = N \ d, c);
      if (q <= 5, c = W[min(q, 4)]
	        , c = F[sqrtint(q)] + V[2] * F[sqrtint(q\2)]
	            + V[3] * F[sqrtint(q\3)] + V[6] * F[sqrtint(q\6)]);
      S += c * t));
  return (bitprecision(S, B));
}
