/* Tridiagonal QL Implicit algorithm (TQLI) */

/* a << b to accuracy 2^(-B) ? */
Negligible(a, b, B) = b && exponent(b) - exponent(a) > B;

Getm(d, e, N, j, B) =
{
  for (m = j, N - 1,
    if (!e[m] || Negligible(e[m], d[m], B)
              || Negligible(e[m], d[m+1], B), return (m)));
  return (N);
}

Tqli(d, e) =
{ my(B = getlocalbitprec(), N = #d, n = if (!d, N >> 1, N), m);

  localbitprec(B + 32);
  e = concat(e, 0);
  for (j = 1, n,
    while ((m = Getm(d, e, N, j, B)) != j,
      my(p = 0, s = 1, c = 1, g, r);
      g = (d[j+1] - d[j]) / (2 * e[j]);
      r = sqrt(1 + g^2);
      g = d[m] - d[j] + e[j] / if (g < 0, g - r, g + r);
      forstep (i = m - 1, j, -1,
        my(f = s * e[i], b = c * e[i]);
        e[i+1] = r = sqrt(f^2 + g^2);
        if (exponent(r) < -B, d[i+1] -= p; e[m] = 0; next(2));
        s = f / r; c = g / r; g = d[i+1] - p;
        r = (d[i] - g) * s + 2 * c * b;
        p = s * r; d[i+1] = g + p; g = c * r - b);
      d[j] -= p; e[j] = g; e[m] = 0));
  return (bitprecision(vecsort(d[1..n]), B));
}
