/* If s != 1, compute zeta(s, x), otherwise compute -psi(x).
 * In particular, if x = 1 compute zeta(s), and Euler's
 * constant if s = 1. */
ZetaHurwitz(s, x) =
{ my(B = getlocalbitprec(), a = -s, ra = real(a));
  my(rndra = round(ra), k, N, Nx, N2, S1, S2);

  if (x <= 0, error("x <= 0 in ZetaHurwitz"));
  if (rndra >= 0 && exponent(a - rndra) < 16 - B,
    /* s ~ negative integer */
    k = ra + 1;
    k = 2 * ceil(k/2);
    N = 1;
 , /* else */
    localbitprec(64);
    my(c = if (type(a) == "t_INT", 1, 20 * log(B)));
    k = max(ra + 3/2, B * log(2) / (1 + lambertw(Pi / c)));
    k = 2 * ceil(k/2); /* ensure that k is even */
    my(C = 2 * binomial(a, k+1) * bernreal(k+2) / (k+2));
    N = max(1, ceil((abs(C) * 2.^B)^(1 / (k+1-ra)) + 1 - x)));
  bernvec(k \ 2); /* initialize Bernoulli cache */
  Nx = N + x - 1; N2 = Nx^(-2);
  S1 = if (x == 1, dirpowerssum(N, a)
                 , sum(m = 0, N - 1, (m + x)^a, 0.));
  S2 = 0;
  forstep (j = k, 2, -2,
    my(aj = a - j);
    S2 = bernreal(j)/j + aj * (aj+1) / (j*(j+1)) * S2 * N2);
  S2 = 1/2 + S2 * a / Nx;
  if (s == 1, S1 -= log(Nx), S2 += Nx / (a + 1));
  return (S1  - Nx^a * S2);
}
