FindK(sig, T) =
{ my(B = getlocalbitprec());
  localbitprec(32);
  my(E = (B + 1.5 * sig) * log(2), A = log(T) - 1.528);
  return (ceil(2 * Solvemulneglog(A, E - (A + 0.5) / 2)));
}

/* F(z) = sum_n C[n+1] z^n; W[n+1] = (-i Pi/2)^n */
GetC(K, T) =
{ my(B = getlocalbitprec(), M = 3 * K, V, W, W1, W2, VE);
  my(E, VE, C, c1, c2);

  localbitprec(B + 5 * M);
  c1 = -I / sqrt(2); c2 = exp(3 * I * Pi / 8) / 2;
  V = vector(2 * M + 1); V[2 * M + 1] = 1 / factorial(2 * M);
  forstep (j = 2 * M, 1, -1, V[j] = j * V[j + 1]);
  W = powers(I * Pi / 2, 2 * M);
  W1 = vector(M + 1, n, V[2 * n - 1] * W[2 * n - 1]);
  W2 = vector(M + 1, n, c1 * W1[n] + c2 * V[n] * W[n]);
  E = eulervec(M);
  VE = vector(M + 1, n, E[n] * W1[n] << (2 * (n - 1)));
  C = Vec(Ser(VE) * Ser(W2));
  forstep (n = 2, #W, 2, W[n] = - W[n]);
  return ([C, W]);
}

/* D[k+1][j+1] = d_j^{(k)}; K >= 2 */
GetD(K, sig) =
{ my(D = vector(K+1), e = 2.0 - 4*sig);
  my(A = vector((3*K) \ 2)); /* A[d] = (-1)^(d-1) (2*d)!/d! */
  A[1] = 2; for (d = 2, #A, A[d] = -A[d-1] * (4*d - 2));
  D[1] = [1]; D[2] = [1 / 12, e / 4];
  for (k = 2, K,
    my(k3 = 3 * k, J = (k3-1) \ 2, Ds = vector(k3 \ 2 + 1));
    Ds[1] = D[k][1] / (k3 << 2);
    Ds[2] = (D[k][2] + e * D[k][1]) / ((k3 - 2) << 2);
    for (j = 2, J - 1,
      my(c = k3 - 2*j);
      Ds[j+1] = (D[k][j+1] + e * D[k][j]) / (c << 2)
                - (c + 1) * D[k][j-1]);
    my(c = k3 - 2*J);
    Ds[J+1] = e * D[k][J] / (c << 2) - (c + 1) * D[k][J-1];
    if (k % 2 == 0,
      Ds[J+2] = sum(i = 0, J, A[J-i+1] * Ds[i+1]));
    D[k+1] = Ds);
  return (D);
}

/* Re(s) >= 1 / 2, Im(s) >= 12 * B */
RiemannSiegel(s) =
{ my(B = getlocalbitprec(), sig = real(s), T = imag(s));
  my(K = FindK(sig, T), F, D, S, a, N);

  localbitprec(B + 32);
  T = bitprecision(T, B + 32);
  a = sqrt(T / (2 * Pi)); N = floor(a);
  my([C, W] = GetC(K, T), p = 1 - 2 * (a - N), v, b, P);
  v = powers(p^2, #C-1); v = vector(#v, n, v[n] * C[n]);
  P = powers(1 / p, 3 * K);
  F = vector(3 * K + 1, j,
    if (j != 1, v = vector(#v, n, v[n] * (2*n-j)));
    vecsum(v) * P[j]); /* F[i+1] = F^(i)(p) */
  b = powers(1 / (Pi^2 * a), K);
  D = GetD(K, sig);
  S = sum(k = 0, K,
	  b[k+1] * sum(j = 0, (3*k) \ 2,
                       W[j+1] * D[k+1][j+1] * F[3*k-2*j+1]));
  dirpowerssum(N, -s) +
    (-1)^(N - 1) * S * exp(-s * log(a) + I * (T / 2 + Pi / 8));
}

Getchis(s) =
{ my(B = getlocalbitprec(), T = imag(s), G);

  localbitprec(32);
  localbitprec(B + exponent(T * (log(T) - 1)));
  G = lngamma((1 - s) / 2) - lngamma(s / 2);
  localbitprec(B + 32);
  return (exp((s - 1/2) * log(Pi) + G));
}

ZetaRiemannSiegel(s) =
{ my(B = getlocalbitprec(), X, z1, z2);
  my(T = imag(s), flconj = T < 0, h = real(s) - 1/2);

  s = bitprecision(s, B + 32);
  if (flconj, s = conj(s));
  X = Getchis(s);
  if (h < 0, s = 1 - conj(s)); /* use functional equation */
  if (abs(T) < 12 * B, error("T too small in RiemannSiegel"));
  z1 = RiemannSiegel(s);
  z2 = if (h, RiemannSiegel(1 - conj(s)), z1);
  if (h < 0, [z1, z2] = [z2, z1]);
  z1 += X * conj(z2);
  return (if (flconj, conj(z1), z1));
}
