/* Given admissible w = 0e_2....e_{k-1}1, compute a,b,v such
 * that w=0{1}_{b-1}v{0}_{a-1}1 with v empty or admissible.
 * Input: binary vector evec=[e_2,...,e_{k-1}]
 * Output: [a, b, minit, mmid, mfin]. */
Fdi(v1, v2) = fromdigits(concat(v1, v2), 2) + 2;
Findabm(m) =
{ my(minit, mmid, mfin);
  if (!m, return ([1, 1, 2, 1, 2]));
  my(v = valuation(m, 2), a = 1 + v, b);
  m >>= v;
  if (hammingweight(m+1) == 1,
    my(e = exponent(m+1)); \\ now m = {1}_e
    mfin = if (a > 1, 2^(a-2) + 2, 2);
    return ([a, e + 1, 2^e + 1, 1, mfin]));
  my(evec = binary(m), n = #evec);
  for (j = 2, n, \\ b = index of first 0
    if (!evec[j], b = j; break));
  minit = Fdi(vector(b, j, 1), evec[b..n-1]);
  mmid  = Fdi([1], evec[b+1..n-1]);
  mfin  = Fdi([1], evec[b+1..n]) << v;
  return ([a, b, minit, mmid, mfin]);
}

/* For 1 <= m < 2^{k-1}, 1 <= n <= N + 1, Z[m+2][n] is
 *   zeta(w)_{n-1}, w corresponding to m (if v=0y1, m=1y).
 * Z[1] is zeta(emptyset)_{n-1},
 * Z[2] is zeta({0})_{n-1}=zeta({1})_{n-1} for n >= 2. */

Zetamultfill(k, N) =
{ my(K = 2^(k - 1), Z, binvec, vabm, vn);

  binvec = vector(N+1); binvec[1] = 1;
  for (n = 1, N, binvec[n+1] = (2*(2*n-1) * binvec[n]) / n);
  Z = vector(K + 1);
  Z[1] = vector(N+1, j, 1. / binvec[j]);
  Z[2] = vector(N+1, j, if (j > 1, Z[1][j] / (j-1), 0));
  vn = vector(N, n, powers(1. / n, k)); \\ vn[n][a+1] = n^(-a)
  vabm = vector(2^(k-2), m, Findabm(m-1));
  for (k1 = 2, k, /* assume length < k1 filled */
    for (m = 2^(k1-2), 2^(k1-1) - 1,
      my([a, b, minit, mmid, mfin] = vabm[m - 2^(k1-2) + 1]);
      my(S = 0);
      Z[m+2] = vector(N+1);
      forstep (n = N, 1, -1,
        S += (Z[minit][n+1] * vn[n][a+1] +
              Z[mfin][n+1]  * vn[n][b+1] +
              Z[mmid][n+1]  * vn[n][a+b+1]);
        Z[m+2][n] = S)));
  return (Z);
}

Zetamultall(k) =
{ my(B = getlocalbitprec(), N = B / 2 + 16, Z);
  localbitprec(B + 32);
  Z = Zetamultfill(k, N);
  return (bitprecision(vector(#Z-2, j, Z[j+2][1]), B));
}
